

function [resOBS,resERR,resEZ,Log_Likelihood]=KALMANQ_FILTER(F0,F1,F2,F11,F12,F22,G,omega,omegaU,DATA);
% March 4, 2013
% Robert Kollmann, robert_kollmann@yahoo.com
% This proc takes the coefficients of a 2nd order 'pruned' DSGE model approximation, and the (Txn) DATA matrix as inputs. 
% Underlying model: 
% S2(t+1)=F0+F1*S2(t)+F2*e(t+1)+F11*polynom2(S1(t))+F12*polynom2ALL(S1(t),e(t+1))+F22*polynom2(e(t+1))
% S1(t+1)=F1*S1(t)+F2*e(t+1)

% S2(t+1): second-order accurate model solution
% S1(t+1): first-order accurate model solution

% Recursion for polynom2(S1(t+1)): 
%  polynom2(S1(t+1))=K11*polynom2(S1(t))+K12*polynom2ALL(S1(t),e(t+1))+K22*polynom2(e(t+1))

% A state-space representation is constructed in terms of 
% Ztilde(t+1)=[S2(t+1);polynom2(S1(t+1));S1(t+1)]
% Ztilde(t+1)=G0+G1*Ztilde(t)+G2*e(t+1)+G12*polynom2ALL(S(t),e(t+1))+G22*polynom2(e(t+1))

% Observation equation:  y(t)=G*S2(t)+u(t), with u(t): i.i.d. measurement  error

% The proc uses a Kalman filter for S2(t), based on a LINEAR updating
% equation:   E{Ztilde(t+1)|I(t+1)}=E{Ztilde(t+1)|I(t)}+psi(t)*[y(t+1)-E{y(t+1)|I(t)}],

% FOR AN EXTENDED UPDATING EQUATION, SEE THE PROC KALMANQ_2_FILTER.m

% The proc returns:
% (a) resOBS: date t estimate of true values of date t observables
% (b) resERR: date t prediction error: y(t)-E(y(t)|I(t-1)  
% (c) resEZ: date t estimate of true 2nd order state: E(S2(t)|I(t)) 
% (d) Log_Likelihood

% February 18, 2013
% Robert Kollmann, robert_kollmann@yahoo.com

if nargin~=10; error('QQQ'); end;

nS=rows(F0);    % number of states
nE=rows(omega); % number of exogenous innovations
nO=rows(G);     % number of observables
nP2=.5*nS*(nS+1); 
nZ=nS+nP2+nS;  % number of elements of Z=[S1; P2(S1); S1]

if rows(F1)~=nS; error('AA1'); end;
if rows(F2)~=nS; error('AA2'); end;
if rows(F11)~=nS; error('AA3'); end;
if rows(F12)~=nS; error('AA4'); end;
if rows(F22)~=nS; error('AA5'); end;
if cols(F2)~=nE;  error('AA6'); end;
if cols(omega)~=nE; error('AA7'); end;
if cols(DATA)~=nO;  error('AA8a'); end;
if rows(DATA)==0;   error('AA8b'); end;
if cols(G)~=nS;     error('AA9'); end;
if cols(F11)~=nP2;  error('AA10'); end;
if cols(F12)~=nE*nS;  error('AA11'); end;
if cols(F22)~=.5*nE*(nE+1);  error('AA12'); end;
if rows(omegaU)~=cols(omegaU); error('AA13'); end;
if rows(G)~=nO;     error('AA14a'); end;
if cols(G)~=nS;     error('AA14b'); end;

if maxaa(eig(F1))>=.999; res=-1e40; return; end;


Gtilde=[G zeros(nO,nP2+nS)];

y=DATA;


% ..........................................................................
% ..........................................................................
% QUADRATIC KALMAN FILTER

P2nS=[];
k=0;
i=1; while i<=nS;
j=1; while j<=nS;
k=k+1;    
if j>=i; P2nS=[P2nS;k]; end;
j=j+1; end;
i=i+1; end;


P2nE=[];
k=0;
i=1; while i<=nE;
j=1; while j<=nE;
k=k+1;    
if j>=i; P2nE=[P2nE;k]; end;
j=j+1; end;
i=i+1; end;

% is A is nSxnS matrix, then A(P2nS) produces a column vector consisting of
% the elements of A on and below the main diagonal


% recursion for P2(S1(t))
K11=[];  K12=[]; K22=[];
i=1; while i<=nS;
j=i; while j<=nS;
WSL=F1(i,:)'*F1(j,:);
WSP=WSL+WSL'-diag(diag(WSL));
K11=[K11; WSP(P2nS)'];
% ...........
DEH=F1(i,:)'*F2(j,:)+F1(j,:)'*F2(i,:);
DEHa=DEH'; K12=[K12; DEHa(:)'];
% ...........
BRE=F2(i,:)'*F2(j,:);
KLE=BRE+BRE'-diag(diag(BRE));
K22=[K22; KLE(P2nE)'];
j=j+1; end;
i=i+1; end;


% ..................................................................
% March14 2013
% Observation equation
% y(t)=GAM*S2(t)+u(t);   u(t): measurement error
% Approximate polynom2(y(t)) by polynom2(GAM*S1(t)+u(t))
% polynom2(GAM*S1(t)+u(t))=H11_WSL*polynom2(S1(t))+
%      H12_WSL*polynom2ALL(S1(t),u(t))+ H22_WSL*polynom2(u(t))
% H22_WSL=I;

E_WSL=eye(nO);
H11_WSL=[]; H12_WSL=[]; H22_WSL=eye(.5*nO*(nO+1));
i=1; while i<=nO;
j=i; while j<=nO;
STH=G(i,:)'*G(j,:);
EUP=STH+STH'-diag(diag(STH));
H11_WSL=[H11_WSL; EUP(P2nS)'];
% ...........
DEH_WSL=G(i,:)'*E_WSL(j,:)+G(j,:)'*E_WSL(i,:);
DEHa_WSL=DEH_WSL'; H12_WSL=[H12_WSL; DEHa_WSL(:)'];
j=j+1; end;
i=i+1; end;

L1_WSL=zeros(nO+.5*nO*(nO+1),nS+nP2+nS);
L1_WSL(1:nO,1:nS)=G;
L1_WSL(nO+1:end,nS+1:nS+nP2)=H11_WSL;

L2_WSL=[eye(nO); zeros(.5*nO*(nO+1),nO)];
L12_WSL=[zeros(nO,nS*nO); H12_WSL];
L22_WSL=[zeros(nO,.5*nO*(nO+1));H22_WSL];

% ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% covariance of polynom2(u): VP2u
resI_WSL=[];
i=1; while i<=nO;
j=i; while j<=nO;
resI_WSL=[resI_WSL; [i j]];
j=j+1; end;
i=i+1; end;

VP2u_A_WSL=zeros(.5*nO*(nO+1),.5*nO*(nO+1));
i=1; while i<=.5*nO*(nO+1);
j=i; while j<=.5*nO*(nO+1);
r1=resI_WSL(i,1); r2=resI_WSL(i,2);
s1=resI_WSL(j,1); s2=resI_WSL(j,2);
fr1=zeros(nO,1); fr1(r1)=1; 
fr2=zeros(nO,1); fr2(r2)=1; 
fs1=zeros(nO,1); fs1(s1)=1; 
fs2=zeros(nO,1); fs2(s2)=1; 
VP2u_A_WSL(i,j)=2*trace(.5*((fr1*fr2')+(fr2*fr1'))*omegaU*.5*((fs1*fs2')+(fs2*fs1'))*omegaU);
j=j+1; end;
i=i+1; end;
VP2u=(VP2u_A_WSL+VP2u_A_WSL')-diag(diag(VP2u_A_WSL));

%T=1000000;
%u=randn(T,nO)*sqrt(omegaU);
%u=randn(T,nO)*cholPSD(omegaU);
%u2=polynom2(u);

P2nU=[];
k=0;
i=1; while i<=nO;
j=1; while j<=nO;
k=k+1;    
if j>=i; P2nU=[P2nU;k]; end;
j=j+1; end;
i=i+1; end;

% Mean of polynom2(u)
EP2u=omegaU(P2nU);


% .............................................................

% The recursion for Z(t)=[S2(t);polynom2(S1(t));S1(t)]
G0=[F0;zeros(nP2+nS,1)];
G1=zeros(nZ,nZ);
G1(1:nS,1:nS)=F1; G1(1:nS,nS+1:nS+nP2)=F11; 
G1(nS+1:nS+nP2,nS+1:nS+nP2)=K11;
G1(nS+nP2+1:end,nS+nP2+1:end)=F1;
G2=[F2;zeros(nP2,nE);F2];
G12=[F12;K12;zeros(nS,nS*nE)];
G22=[F22;K22;zeros(nS,.5*nE*(nE+1))];

%tabulate1(G22)
%space(1)

% .............................................................
% covariance of polynom2(e): VP2e
resI=[];
i=1; while i<=nE;
j=i; while j<=nE;
resI=[resI; [i j]];
j=j+1; end;
i=i+1; end;

%sA=(1:1:nE)'*ones(1,nE); 
%sB=ones(nE,1)*(1:1:nE); 
%resIa=[sB(P2nE) sA(P2nE)];
%if maxaa(resIa-resI)>1e-8; error('AA');


VP2e_A=zeros(.5*nE*(nE+1),.5*nE*(nE+1));
i=1; while i<=.5*nE*(nE+1);
j=i; while j<=.5*nE*(nE+1);
r1=resI(i,1); r2=resI(i,2);
s1=resI(j,1); s2=resI(j,2);
fr1=zeros(nE,1); fr1(r1)=1; 
fr2=zeros(nE,1); fr2(r2)=1; 
fs1=zeros(nE,1); fs1(s1)=1; 
fs2=zeros(nE,1); fs2(s2)=1; 
VP2e_A(i,j)=2*trace(.5*((fr1*fr2')+(fr2*fr1'))*omega*.5*((fs1*fs2')+(fs2*fs1'))*omega);
j=j+1; end;
i=i+1; end;
VP2e=(VP2e_A+VP2e_A')-diag(diag(VP2e_A));

% Mean of polynom2(e)
EP2e=omega(P2nE);

% .......................................................................
% Unconditional covariance of  Z(t)=[S2(t);polynom2(S1(t));S1(t)]
% Unconditional covariance of S1(t)
VS1=ardouble(F1,F2*omega*F2');  % E(S1*S1')
if maxaa(VS1-(F1*VS1*F1'+F2*omega*F2'))>1e-6; 
    save klm7; res=-1e40; return; end;

VZtilde_A=ardouble(G1,G2*omega*G2'+G12*kron(VS1,omega)*G12'+G22*VP2e*G22'); % Var(Ztilde)
if maxaa(VZtilde_A-(G1*VZtilde_A*G1'+G2*omega*G2'+G12*kron(VS1,omega)*G12'+G22*VP2e*G22'))>1e-6; 
    save klm8; res=-1e40; return; end;
     

Vy2P2y1=L1_WSL*VZtilde_A*L1_WSL'+L2_WSL*omegaU*L2_WSL'+L12_WSL*kron(VS1,omegaU)*L12_WSL'+L22_WSL*VP2u*L22_WSL';
% covariance matrix of [G*S2(t)+u(t);P2(G*S1(t)+u(t))];

% ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

%save klm777;
%error('AAAA');


% ........................
fasterV=0; 
if fasterV==1;
VZtilde_B=zeros(nS+nP2+nS,nS+nP2+nS);
VZtilde_B(1:nS+nP2,1:nS+nP2)=ardouble(G1(1:nS+nP2,1:nS+nP2),G2(1:nS+nP2,:)*omega*G2(1:nS+nP2,:)'+ ...
G12(1:nS+nP2,:)*kron(VS1,omega)*G12(1:nS+nP2,:)'+G22(1:nS+nP2,:)*VP2e*G22(1:nS+nP2,:)'); 
VZtilde_B(:,nS+nP2+1:end)=[VS1;zeros(nP2,nS);VS1];
VZtilde_B(nS+nP2+1:end,:)=[VS1;zeros(nP2,nS);VS1]';
end;
% ........................


EZtilde=inv(eye(nZ)-G1)*(G0+G22*omega(P2nE));   %  E(Ztilde)


% .......................................................................
% The QUADRATIC Kalman filter
VZ=VZtilde_A;   %  Var(Ztilde(0)|I(0))
EZ=EZtilde;     %  E(Ztilde(0)|I(0)

resOBS=[]; resERR=[]; resEZ=[]; resVZ=[]; resCovYErr=[];  LOG_L=[];

t=1; while t<=rows(y);
if (t/5000-ceil(t/5000))==0; display(t);  end;  

Vint1=G12*kron(EZ(nS+nP2+1:end),omega)*G2';
Vint2=G12*kron(VZ(nS+nP2+1:end,nS+nP2+1:end)+EZ(nS+nP2+1:end,:)*EZ(nS+nP2+1:end,:)',omega)*G12';
VZp=G1*VZ*G1'+G2*omega*G2'+Vint1+Vint1'+Vint2+G22*VP2e*G22';  % Var(Ztilde(t)|I(t-1))

CovYerr=Gtilde*VZp*Gtilde'+omegaU;
%resCovYErr=[resCovYErr; CovYerr(:)'];

%kpsi=inv(Gtilde*VZp*Gtilde'+omegaU);           
kpsi=inv(CovYerr);           

if det(kpsi)<=0; error('AAA'); end;

psi=VZp*Gtilde'*kpsi;

EZp=G0+G1*EZ+G22*EP2e;                % E(Ztilde(t)|I(t-1))

EZpp=EZp+psi*(y(t,:)'-Gtilde*EZp);    % E(Ztilde(t)|I(t))

RK=eye(rows(VZp))-psi*Gtilde;
VZpp=RK*VZp*RK'+psi*omegaU*psi';      % Var(Ztilde(t)|I(t))
% Formula that ensures that Var(Ztilde(t)|I(t)) is positive definite
% see Carraro and Sartore (1987), Annales d'Economie et de Statistique,
% p.439

%VZppOLD=VZp-VZp*Gtilde'*kpsi*Gtilde*VZp';          % Var(Ztilde(t)|I(t))
%if maxaa(VZpp-VZppOLD)>1e-8; error('AA'); end;

resOBS=[resOBS; (Gtilde*EZpp)'];        % date t estimate of true values of date t observables
resERR=[resERR; (y(t,:)'-Gtilde*EZp)']; % date t prediction error: y(t)=E(y*t)|I(t-1)  
resEZ=[resEZ; EZpp(1:nS)'];             % E(S2(t)|I(t)) 
%resVZ=[resVZ;VZpp(:)'];


logh=.5*log(det(kpsi))-.5*(y(t,:)'-Gtilde*EZp)'*kpsi*(y(t,:)'-Gtilde*EZp)-.5*nO*log(2*pi);
LOG_L=[LOG_L; logh];


VZ=VZpp;
EZ=EZpp;

t=t+1; end;

Log_Likelihood=sum(LOG_L);

