

clear all;
clc
% 21.2.2013
% Robert Kollmann, robert_kollmann@yahoo.com

% RBCmodKQ_web.m  (see also RBCmodf.m)

% This program solves a simple closed economy RBC model to second order,
% and it applies a QUADRATIC KALMAN FILTER, and a PARTICLE FILTER to
% estimate the latent states of the model. 

% The model is defined in RBSmodf.m
% first- and second-order solutions are computed using gensys2 (Sims (2000))

global BETA CRRA ELS KPSI DEPRECss PSIss THss RHOth RHOpsi  
global omega omegaU

global DATA_KALMANQ  G_KALMANQ  



nS=7;   % number of states
nO=4;   % number of observables
nE=2;   % number of exogenous variables
nP2=.5*nS*(1+nS); % number of squares and cross-products of states
nZ=nS+nP2+nS;  % number of elements of Z=[S1; P2(S1); S1]

% PARAMETERS THAT ARE CALIBRATED
BETA=.99; 
KPSI=.3;     % capital share

DEPRECss=.025; % capital depreciation rate

ssU=4;       % standard deviation of measurement error (in %)
omegaU=((ssU/100)^2)*eye(nO);  % covariance matrix of measurement errors


% Selection matrix for observation equation:  
% observables = G*(state vector) + meas. error
% See RBCmodf.m for state variables: k,gdp,C,I,Lab,th,psi (psi=taste shock)
G=zeros(nO,nS); 
G(1,2)=1;
G(2,3)=1;
G(3,4)=1;
G(4,5)=1;
Gtilde=[G zeros(nO,nP2+nS)];
G_KALMANQ=G;


%       (1)      (2)     (3)     (4)     (5)     (6)     (7)
fghCOL=['k    ';'gdp  ';'C    ';'I    ';'Lab  ';'th   ';'psi  '];
fghROW=['k      ','gdp    ','C      ','I      ','Lab    ','th     ','psi    '];
display('Observables:  ');
tabulate1(fghCOL(sum(G)'==1,:))


randn('seed',5001552); 


resXSIMS5=[];
resSTD_QML=[];
resSTD_QML_HESS=[];

resRMSE_KQ=[]; resRMSE_PF=[]; resRMSE_LIN=[];
resMAX_KQ=[];  resMAX_PF=[];  resMAX_LIN=[];
resCORR_KQ=[]; resCORR_PF=[]; resCORR_LIN=[];


% ========== ITERATE OVER SIMULATED DATA SETS ==============
zksed=1; while zksed<=50;    % 50 simulation runs

display('=========================');    
tabulate1('zksed:  ',zksed)
zeit;
display('=========================');


CRRA=10;   % risk aversion
ELS=4;     % labor supply elasticity 
RHOth=.99; RHOpsi=.99;   % autocorr. of exogenous variables

% standard deviations of shock innovations (in %)
ssTH=1; ssPSI=.05;    % small shocks case
ssTH=20; ssPSI=1;     % big shocks case

% parameter vector (used for quai-maximum likelihood estimation)
PARAMV=[CRRA;ELS;RHOth;RHOpsi;ssTH;ssPSI];

% covariance matrix of exogenous innovations
omega=zeros(nE,nE);
omega(1,1)=(ssTH/100)^2;
omega(2,2)=(ssPSI/100)^2;

% steady state
LABss=1;
Yss=1;
Kss=KPSI/((1-BETA)/BETA+DEPRECss);
Css=1-DEPRECss*Kss;
Iss=DEPRECss*Kss;
PSIss=(1-KPSI)*Css^-CRRA;
THss=Kss^-KPSI;

sig=1;  % scale parameter for exogenous shocks

xss1=log([Kss;Yss;Css;Iss;LABss;THss;PSIss]);
xss3=[0;0];

if rows(omega)~=rows(xss3); error('LLL1'); end;
if sum(isnan(xss1))>0; display('xss1 not defined'); save klm001; error('AAA'); end;
if rows(omega)~=rows(xss3); error('LLL1'); end;


c=RBCmodf([xss1;xss1;xss3]);

if sum(isnan(c))>0; display('SS not correct 1'); error('AAA1'); end;
if maxaa(c)>1e-7; display('SS not correct 2');   error('AAA2'); end; 


% 
% Pi matrix used by gensys2 (specifies what model equations are
% expectationl equations) 

Pi=zeros(rows(c),1);
Pi(1:cols(Pi),:)=eye(cols(Pi));


% ..................................................................................
% THIS COMPUTES FIRST- AND SECOND ORDER ACCURATE MODEL SOLUTIONS, USING
% GENSYS2
% first and second derivatives of model equations, at steady state
[KD,KDD,KDDALL]=pregen('RBCmodf',xss1,xss3); 

%tabulate1([maxaa(KD{1});maxaa(KD{2});maxaa(KD{3});maxaa(KDD{1,1});maxaa(KDD{1,2}); ...
%   maxaa(KDD{1,3});maxaa(KDD{2,2});maxaa(KDD{2,3});maxaa(KDD{3,3})])



%[FD,FDD,M11,M2,C,q,zs,zu,v,gev,eu,six,usix]=gensys2(KD,KDD,c,Pi,omega,pick);
[FD,FDD,M11,M2,C,q,zs,zu,v,gev,eu,six,usix]=gensys2(KD,KDD,c,Pi,omega);
pick=zs;

% This proc takes the output of gensys2, and produces the coefficients
% needed by KalmanQ (see paper): 
% w(t+1)=F0+F1*w(t)+F2*e(t+1)+F11*P(w(t))+F12*KRON(w(t),e(t+1))+F22*P(e(t+1))
% As in paper, P(x)=vector of squares and cross products of elements of
% column vector x. 
[F0,F1,F2,F11,F12,F22]=gensys2_TO_KALMANQ(FD,FDD,M11,M2,pick,zu,sig);
% .................................................



% ========================================================================================
% ========================================================================================
% NEXT COMPUTE MOMENTS PREDICTED BY 1st & 2nd order models
PRED_MOM=1;
if PRED_MOM==1;
    
% Predicted moments (unfiltered series)
[VZtilde,VZtilde1,EZtilde,VS,VS1]=gensys2_moments(F0,F1,F2,F11,F12,F22,omega);
% Predicted moments (HP filtered series)
[HP_VZtilde,HP_VZtilde1,HP_VS,HP_VS1]=gensys2_HPmoments(F0,F1,F2,F11,F12,F22,omega);

% MnHP1: Non-HP filtered series: (std with 2nd order approx)/(std with lin approx);
% MnHP2: Non-HP filtered series:  correlation between 1st & 2nd order approx;
MnHP1=sqrt(diag(VZtilde(1:nS,1:nS))./diag(VS));  
MnHP2=diag(VZtilde(1:nS,nS+nP2+1:end))./sqrt(diag(VZtilde(1:nS,1:nS)).*diag(VZtilde(nS+nP2+1:end,nS+nP2+1:end)));

% MHP1: HP filtered series: (std with 2nd order approx)/(std with lin approx);
% MHP2: HP filtered series:  correlation between 1st & 2nd order approx;
MHP1=sqrt(diag(HP_VZtilde(1:nS,1:nS))./diag(HP_VS));  
MHP2=diag(HP_VZtilde(1:nS,nS+nP2+1:end))./sqrt(diag(HP_VZtilde(1:nS,1:nS)).*diag(HP_VZtilde(nS+nP2+1:end,nS+nP2+1:end)));

send;
zeit;
display('KQ: quadr. Kalman;  LIN: Lin Kalman')
display('BETA   KPSI   CRRA   ELS  DEPRECss RHOth  RHOpsi sTH%   sPSI%  sU%') ; 
disp([num2str([BETA   KPSI   CRRA   ELS   DEPRECss RHOth RHOpsi ssTH ssPSI ssU],'%7.3f')]);
display('OBSERVABLES:');
tabulate1(fghCOL(sum(G)==1,:))
display('.....................');


display('relSTD: std(quadr approx)/std(lin approx); Corr: corr(Quadr,Lin)');
display('      Non-HP filt   HP filtered'); 
display('      relSTD Corr   relSTD Corr'); 
tabulate1(fghCOL,[MnHP1 MnHP2  MHP1 MHP2]);

% ........................................................................
% MOMENTS GENERATED BY THE MODEL

resSTD_S2=[]; resSTD_HP_S2=[]; 
resSTD_S1=[]; resSTD_HP_S1=[]; 

i=0; while i<=rows(omega);
omegaFDF=zeros(size(omega));
if i==0; omegaFDF=omega; end;
if i>=1; omegaFDF(i,i)=omega(i,i); end;

% gensys2_moments: computes moments of 1st & 2nd order solutions
% gensys2_moments: computes moments of HP filteres variables (1st & 2nd order solutions)
[VZtilde,VZtilde1,EZtilde,VS,VS1]=gensys2_moments(F0,F1,F2,F11,F12,F22,omegaFDF);
[HP_VZtilde,HP_VZtilde1,HP_VS,HP_VS1]=gensys2_HPmoments(F0,F1,F2,F11,F12,F22,omegaFDF);
if min(diag(VZtilde))<-1e-8; error('negative variance'); end;
if min(diag(HP_VZtilde))<-1e-8; error('negative HPvariance'); end;
if min(diag(VS))<-1e-8; error('negative variance'); end;
if min(diag(HP_VS))<-1e-8; error('negative HPvariance'); end;
VZtilde=abs(VZtilde); HP_VZtilde=abs(HP_VZtilde);
VS=abs(VS); HP_VS=abs(HP_VS);

resSTD_S2=[resSTD_S2 sqrt(diag(VZtilde(1:nS,1:nS)))];
resSTD_HP_S2=[resSTD_HP_S2 sqrt(diag(HP_VZtilde(1:nS,1:nS)))];
resSTD_S1=[resSTD_S1 sqrt(diag(VS))];
resSTD_HP_S1=[resSTD_HP_S1 sqrt(diag(HP_VS))];
i=i+1; end;

display('.........................');
display('Std of S2: col 1: ALL shocks; cols 2-n: variance SHARES of indiv shocks')
display('... NOT FILTERED ...');
tabulate1(fghCOL,[resSTD_S2(:,1)    (resSTD_S2(:,2:end)./(resSTD_S2(:,1)*ones(1,nE))).^2]);
display('... HP FILTERED ...');
tabulate1(fghCOL,[resSTD_HP_S2(:,1) (resSTD_HP_S2(:,2:end)./(resSTD_HP_S2(:,1)*ones(1,nE))).^2]);
space(1);

%error('AAA');

display('.........................');
display('Std of S2: col 1: ALL shocks; cols 2-n: Std with just indiv shocks')
display('... NOT FILTERED ...');
tabulate1(fghCOL,[resSTD_S2]);
display('... HP FILTERED ...');
tabulate1(fghCOL,resSTD_HP_S2);

display('.........................');
display('Std of S1: col 1: ALL shocks; cols 2-n: Std with just indiv shocks')
display('... NOT FILTERED ...');
tabulate1(fghCOL,[resSTD_S1]);
display('... HP FILTERED ...');
tabulate1(fghCOL,resSTD_HP_S1);


% ======================================================
% IMPULSE RESPONSES

i=1; while i<=cols(omega);
exo=zeros(201,cols(omega));
exo(1,i)=.1;
%Pwsl1=arsimqL_P(EZtilde(1:nS),exo,FD,FDD,M11,M2,pick,zu,sig);
%Pwsl0=arsimqL_P(EZtilde(1:nS),0*exo,FD,FDD,M11,M2,pick,zu,sig);
%Pwsl=(Pwsl1(2:end,1:nS)-Pwsl0(2:end,1:nS));
[Pwsl1,S1wsl1,S1Pwsl1]=arsimqL_STACK(EZtilde(1:nS),zeros(nS,1),exo,F0,F1,F2,F11,F12,F22);
[Pwsl0,S1wsl0,S1Pwsl0]=arsimqL_STACK(EZtilde(1:nS),zeros(nS,1),0*exo,F0,F1,F2,F11,F12,F22);
Pwsl=(Pwsl1(:,1:nS)-Pwsl0(:,1:nS));

display('......................................');
horiz=['  0 ';'  1 ';'  2 ';'  4 ';' 10 ';' 20 ';'100 ';'200 '];
display(['      ',fghROW]);
display('QUADRATIC IMPULSE RESPONSE');
tabulate1(horiz,Pwsl([1;2;3;5;11;21;101;201],:))
display('.........');
display('LINEAR IMPULSE RESPONSE');
tabulate1(horiz,S1wsl1([1;2;3;5;11;21;101;201],:))
space(2);
i=i+1; end;

fin;

end;   % end if PRED_MOM==1
% ========================================================================================
% ========================================================================================





% .........................................................................
% ...........................SIMULATED DATA ................................

T=500;   % length of one simulation run
T=100;


e=randn(T,cols(omega))*chol(omega);   % innovations to exogenous variables
u=randn(T,nO)*chol(omegaU);           % measurement errors
z0=zeros(rows(xss1),1);

[VZtilde,VZtilde1,EZtilde,VS,VS1]=gensys2_moments(F0,F1,F2,F11,F12,F22,omega); 
S1init=zeros(nS,1);
S2init=EZtilde(1:nS,:);
[S2,S1,S1P]=arsimqL_STACK(S2init,S1init,e,F0,F1,F2,F11,F12,F22);
% arsimqL_STACK: Proc for simulating the 2nd order accurate model solution
% e=[e(1)';e(2)';...;e(T)'];
% S2: second-order accurate simulated series: [S2(1)';S2(2)';...;S2(T)']
% S1: first-order accurate simulated series:  [S1(1)';S1(2)';...;S1(T)']
% S1P: one-period ahead predictions of 1st-order accurate series:
% [E{S1(2)|I(1)}';E{S1(3)|I(2)}';...;E{S1(T+1)|I(T)}']

tabulate1(std(S2)./std(S1))
tabulate1(correl2(S1,S2))


% ............................................
CHECK_EQN=1;
if CHECK_EQN==1;
% CHECKING EQUATIONS USING SIMULATED SERIES (1st order accurate)
t1=-S1(:,2)+Css*S1(:,3)+Iss*S1(:,4); % definition of gdp
%t2=-(S1(:,7)+(1/ELS)*S1(:,5))+(-CRRA*S1(:,3)+S1(:,6)+KPSI*[S1init(1);S1(1:end-1,1)]-KPSI*S1(:,5)); 
t2=-((1/ELS)*S1(:,5))+(-CRRA*S1(:,3)+S1(:,6)+KPSI*[S1init(1);S1(1:end-1,1)]-KPSI*S1(:,5)); 
%Labor supply
t3=-S1(:,1)*Kss+(1-DEPRECss)*[S1init(1);S1(1:end-1,1)]*Kss+S1(:,4)*Iss; % Law of motion of K
t4=-S1(:,2)+S1(:,6)+KPSI*[S1init(1);S1(1:end-1,1)]+(1-KPSI)*S1(:,5); % Aggregate prod. fn
t5=-S1(:,6)+RHOth*[S1init(6);S1(1:end-1,6)]+e(:,1);  % law of motion of TFP
t6=-S1(:,7)+RHOpsi*[S1init(7);S1(1:end-1,7)]+e(:,2); % law of motion of labor suplly shock
t7=S1(:,7)+-CRRA*(S1P(:,3)-S1(:,3))+((1-BETA)+BETA*DEPRECss)*(S1P(:,6)+(KPSI-1)*S1(:,1)+(1-KPSI)*S1P(:,5));
test1=[t1 t2 t3 t4 t5 t6 t7];
if maxaa(test1)>1e-6; error('AAA'); end;
end;  % end if CHECK_EQN==1;
% ............................................

%z0=zeros(rows(xss1),1); w2=arsimqL(z0,e);
%if maxaa(S2-w2(2:end,1:8))>1e-8; error('AA'); end;

% OBSERVATION EQUATION
y=S2*G'+u;   

%y2=polynom2(y);
%yC=S1*G'+u;   
%y2a=polynom2(S1*G'+u);   


DATA=y;
DATA_KALMANQ=DATA;


%error('AAA');

% KALMANQ_FILTER: the KalmanQ filter
% KALMAN_LIN_FILTER: the standard linear Kalman filter (KalmaL) that assumes that
%   linearized model is the true DGP
% PARTICLE_FILTER: the standard particle filter

[resOBS,resERR,resEZ,Log_Likelihood]=KALMANQ_FILTER(F0,F1,F2,F11,F12,F22,G,omega,omegaU,y);
[resOBS_LIN,resERR_LIN,resEZ_LIN,Log_Likelihood_LIN]=KALMAN_LIN_FILTER(F1,F2,G,omega,omegaU,demean(y));
[resOBS_PF,resERR_PF,resEZ_PF,resNeff,Log_Likelihood_PF,PT]=PARTICLE_FILTER(F0,F1,F2,F11,F12,F22,G,omega,omegaU,100000,y);
% Apply Linear Kalman filter to DEMEANED observables !
%[resOBS2,resERR2,resEZ2,Log_Likelihood2]=KALMANQ_2_FILTER(F0,F1,F2,F11,F12,F22,G,omega,omegaU,y);


%y=S1*G'+u;   
%[resOBS_LIN,resERR_LIN,resEZ_LIN,Log_Likelihood_LIN]=KALMAN_LIN_FILTER(F1,F2,G,omega,omegaU,y);
%[resOBS_PF,resERR_PF,resEZ_PF,resNeff,Log_Likelihood_PF,PT]=PARTICLE_FILTER(0*F0,F1,F2,0*F11,0*F12,0*F22,G,omega,omegaU,100000,y);

space(1)

% Computing RMSEs and Maximum Errors
resRMSE_KQ=[resRMSE_KQ;   rmse(resEZ(:)-S2(:))     rmse(resEZ-S2)];
resRMSE_PF=[resRMSE_PF;   rmse(resEZ_PF(:)-S2(:))  rmse(resEZ_PF-S2)];
resRMSE_LIN=[resRMSE_LIN; rmse(resEZ_LIN(:)-S2(:)) rmse(resEZ_LIN-S2)];

resMAX_KQ=[resMAX_KQ;   maxABS(resEZ(:)-S2(:))     maxABS(resEZ-S2)];
resMAX_PF=[resMAX_PF;   maxABS(resEZ_PF(:)-S2(:))  maxABS(resEZ_PF-S2)];
resMAX_LIN=[resMAX_LIN; maxABS(resEZ_LIN(:)-S2(:)) maxABS(resEZ_LIN-S2)];

resCORR_KQ=[resCORR_KQ;   correl2(resEZ,S2)];
resCORR_PF=[resCORR_PF;   correl2(resEZ_PF,S2)];
resCORR_LIN=[resCORR_LIN; correl2(resEZ_LIN,S2)];


display('RMSE for latent states| (1): KQ; (2): LIN;  (3): PF ');
tabulate1([100*rmse(resEZ(:)-S2(:)) 100*rmse(resEZ-S2); ...
           100*rmse(resEZ_PF(:)-S2(:)) 100*rmse(resEZ_PF-S2); ...
           100*rmse(resEZ_LIN(:)-S2(:)) 100*rmse(resEZ_LIN-S2)])
     
space(1);
display('Max estimation error for latent states');
tabulate1([maxABS(resEZ(:)-S2(:)) maxABS(resEZ-S2); ...
           maxABS(resEZ_PF(:)-S2(:)) maxABS(resEZ_PF-S2); ...
           maxABS(resEZ_LIN(:)-S2(:)) maxABS(resEZ_LIN-S2)])

space(1);
display('Corr. between estimate and true state')
ASZ=['       ';'       ';'       '];
tabulate1(ASZ,[correl2(resEZ,S2); correl2(resEZ_PF,S2); correl2(resEZ_LIN,S2)])

save resZKSED resRMSE_KQ resRMSE_PF resRMSE_LIN resMAX_KQ resMAX_PF resMAX_LIN resCORR_KQ resCORR_PF resCORR_LIN;


zksed=zksed+1; end;

send;
%xy9(resEZ,resEZ_PF,S2)

zeit;
display('Prediction errors: state vars. KQ: quadr. Kalman, PF: particle filt, LIN: Lin Kalman')
display('BETA   KPSI   CRRA   ELS  DEPRECss RHOth  RHOpsi sTH%   sPSI%  sU%') ; 
disp([num2str([BETA   KPSI   CRRA   ELS   DEPRECss RHOth RHOpsi ssTH ssPSI ssU],'%7.3f')]);

disp(['T Particles NrReplics   ',[num2str([T PT rows(resRMSE_KQ)],'%10.0f')]]);


display('OBSERVABLES:');
tabulate1(fghCOL(sum(G)==1,:))
display('.....................');

CVP=['KQ  ';'PF  ';'LIN '];
display(['      ALL    ',fghROW])
display('MEAN RMSEs across simulation runs');
tabulate1(CVP,[mean(resRMSE_KQ);mean(resRMSE_PF);mean(resRMSE_LIN)]);
%display('MEDIAN RMSEs across simulation runs');
%tabulate1(CVP,[median(resRMSE_KQ);median(resRMSE_PF);median(resRMSE_LIN)]);
display('MAX RMSEs across simulation runs');
tabulate1(CVP,[max(resRMSE_KQ);max(resRMSE_PF);max(resRMSE_LIN)]);
display('fraction of RMSE smaller under KQ (across simulation runs)');
tabulate1(CVP([2 3],:),[mean(resRMSE_KQ<resRMSE_PF);mean(resRMSE_KQ<resRMSE_LIN)]);
space(1);
display(['      ALL    ',fghROW])
display('MEAN MAX errors across simulation runs');
tabulate1(CVP,[mean(resMAX_KQ);mean(resMAX_PF);mean(resMAX_LIN)]);
%display('MEDIAN errors across simulation runs');
%tabulate1(CVP,[median(resMAX_KQ);median(resMAX_PF);median(resMAX_LIN)]);
display('MAX of MAX errors across simulation runs');
tabulate1(CVP,[max(resMAX_KQ);max(resMAX_PF);max(resMAX_LIN)]);
display('fraction of MAX errors smaller under KQ (across simulation runs)');
tabulate1(CVP([2 3],:),[mean(resMAX_KQ<resMAX_PF);mean(resMAX_KQ<resMAX_LIN)]);
space(1);
display(['      ALL    ',fghROW])
display('MEAN Correlations between filtered and true states');
tabulate1([CVP  repmat(['       '],3,1)],[mean(resCORR_KQ);mean(resCORR_PF);mean(resCORR_LIN)]);
display('MEDIAN Correlations between filtered and true states');
tabulate1([CVP  repmat(['       '],3,1)],[median(resCORR_KQ);median(resCORR_PF);median(resCORR_LIN)]);
display('MIN Correlations between filtered and true states');
tabulate1([CVP  repmat(['       '],3,1)],[min(resCORR_KQ);min(resCORR_PF);min(resCORR_LIN)]);

fin;


